"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var jsonwebtoken_1 = require("jsonwebtoken");
var _1 = require("../");
describe('Auth Server', function () {
    var ONE_MINUTE = 1000 * 60;
    var ONE_DAY = ONE_MINUTE * 60 * 24;
    var ONE_MONTH = ONE_DAY * 30;
    var ACCESS_TOKEN_COOKIE = 'abc';
    var ACCESS_TOKEN_SECRET = 'password';
    var REFRESH_TOKEN_COOKIE = 'aei';
    var refreshTokens = new Map();
    var AccessToken = /** @class */ (function () {
        function AccessToken(Auth) {
            this.Auth = Auth;
            this.cookie = ACCESS_TOKEN_COOKIE;
        }
        AccessToken.prototype.buildPayload = function (_a) {
            var id = _a.id, companyId = _a.companyId, admin = _a.admin;
            var scope = admin
                ? this.Auth.scope.create(['admin:read', 'admin:write'])
                : '';
            return { id: id, companyId: companyId, scope: scope };
        };
        AccessToken.prototype.create = function (payload) {
            return jsonwebtoken_1.default.sign(payload, ACCESS_TOKEN_SECRET, {
                expiresIn: '20m'
            });
        };
        AccessToken.prototype.verify = function (accessToken) {
            var payload = jsonwebtoken_1.default.verify(accessToken, ACCESS_TOKEN_SECRET, {
                algorithms: ['HS256'],
                clockTolerance: 80 // seconds to tolerate
            });
            // This should never happen cause our payload is a valid JSON
            if (typeof payload === 'string')
                return {};
            return payload;
        };
        AccessToken.prototype.getExpDate = function () {
            return new Date(Date.now() + ONE_MINUTE * 20);
        };
        return AccessToken;
    }());
    var RefreshToken = /** @class */ (function () {
        function RefreshToken(Auth) {
            this.Auth = Auth;
            this.cookie = REFRESH_TOKEN_COOKIE;
        }
        RefreshToken.prototype.create = function (_a) {
            var userId = _a.id;
            return __awaiter(this, void 0, void 0, function () {
                var id;
                return __generator(this, function (_b) {
                    id = Date.now().toString();
                    refreshTokens.set(id, {
                        userId: userId,
                        expireAt: this.getExpDate()
                    });
                    return [2 /*return*/, id];
                });
            });
        };
        RefreshToken.prototype.remove = function (refreshToken) {
            return refreshTokens.delete(refreshToken);
        };
        RefreshToken.prototype.getPayload = function (refreshToken, reset) {
            return __awaiter(this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    reset();
                    return [2 /*return*/, refreshTokens.get(refreshToken)];
                });
            });
        };
        RefreshToken.prototype.getExpDate = function () {
            return new Date(Date.now() + ONE_MONTH);
        };
        return RefreshToken;
    }());
    var authPayload = new _1.AuthPayload({
        uId: 'id',
        cId: 'companyId',
        scope: 'scope'
    });
    var authScope = new _1.AuthScope({
        admin: 'a'
    });
    var authServer = new _1.AuthServer({
        AccessToken: AccessToken,
        RefreshToken: RefreshToken,
        payload: authPayload,
        scope: authScope
    });
    var expiredToken = 'eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ1SWQiOiJ1c2VyXzEyMyIsImNJZCI6ImNvbXBhbnlfMTIzIiwic2NvcGUiOiJhOnI6dyIsImlhdCI6MTUxODE0MTIzNCwiZXhwIjoxNTE4MTQyNDM0fQ.3ZRmx08htMX5KLsv8VhBVD8vjxHzWOiDDli7JXFf83Q';
    // Payload to create a token
    var userPayload = {
        id: 'user_123',
        companyId: 'company_123',
        admin: true
    };
    // Payload got from a token
    var tokenPayload = {
        id: userPayload.id,
        companyId: userPayload.companyId,
        scope: 'a:r:w'
    };
    it('should set a default scope if no scope is used', function () {
        var auth = new _1.AuthServer({
            AccessToken: AccessToken,
            RefreshToken: RefreshToken,
            payload: authPayload
        });
        expect(auth.scope).toBeInstanceOf(_1.AuthScope);
    });
    it('creates an accessToken', function () {
        expect(authServer.createAccessToken(userPayload)).toEqual({
            accessToken: expect.any(String),
            payload: tokenPayload
        });
    });
    it('creates a refreshToken', function () { return __awaiter(_this, void 0, void 0, function () {
        var _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = expect;
                    return [4 /*yield*/, authServer.createRefreshToken(userPayload)];
                case 1:
                    _a.apply(void 0, [typeof (_b.sent())]).toBe('string');
                    return [2 /*return*/];
            }
        });
    }); });
    it('creates both tokens', function () { return __awaiter(_this, void 0, void 0, function () {
        var _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = expect;
                    return [4 /*yield*/, authServer.createTokens(userPayload)];
                case 1:
                    _a.apply(void 0, [_b.sent()]).toEqual({
                        refreshToken: expect.any(String),
                        accessToken: expect.any(String),
                        payload: tokenPayload
                    });
                    return [2 /*return*/];
            }
        });
    }); });
    it('gets the payload for an accessToken', function () { return __awaiter(_this, void 0, void 0, function () {
        var refreshToken, reset, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0: return [4 /*yield*/, authServer.createRefreshToken(userPayload)];
                case 1:
                    refreshToken = _b.sent();
                    reset = function () {
                        // do nothing
                    };
                    _a = expect;
                    return [4 /*yield*/, authServer.getPayload(refreshToken, reset)];
                case 2:
                    _a.apply(void 0, [_b.sent()]).toEqual({
                        userId: userPayload.id,
                        expireAt: refreshTokens.get(refreshToken).expireAt
                    });
                    return [2 /*return*/];
            }
        });
    }); });
    it('Removes a refreshToken', function () { return __awaiter(_this, void 0, void 0, function () {
        var refreshToken;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, authServer.createRefreshToken(userPayload)];
                case 1:
                    refreshToken = _a.sent();
                    expect(authServer.removeRefreshRoken(refreshToken)).toBe(true);
                    expect(authServer.removeRefreshRoken(refreshToken)).toBe(false);
                    expect(authServer.removeRefreshRoken('')).toBe(false);
                    return [2 /*return*/];
            }
        });
    }); });
    describe('Verifies an accessToken', function () {
        it('returns the payload', function () {
            var at = authServer.createAccessToken(userPayload);
            var decodedPayload = authServer.verify(at.accessToken);
            expect(decodedPayload).toEqual(at.payload);
            expect(decodedPayload).toEqual(tokenPayload);
        });
        it('throws if expired', function () {
            expect(function () {
                authServer.verify(expiredToken);
            }).toThrow();
        });
    });
    describe('decodes an accessToken', function () {
        it('Returns the payload', function () {
            var at = authServer.createAccessToken(userPayload);
            var decodedPayload = authServer.decode(at.accessToken);
            expect(decodedPayload).toEqual(at.payload);
            expect(decodedPayload).toEqual(tokenPayload);
        });
        it('Returns null with empty accessToken', function () {
            expect(authServer.decode('')).toBe(null);
        });
        it('Returns null if expired', function () {
            expect(authServer.decode(expiredToken)).toBe(null);
        });
    });
    describe('Gets an accessToken from a request', function () {
        var headers = {
            authorization: 'Bearer ' + expiredToken
        };
        var cookies = (_a = {},
            _a[ACCESS_TOKEN_COOKIE] = 'x' + expiredToken,
            _a);
        it('uses the headers to get the token', function () {
            expect(authServer.getAccessToken({ headers: headers })).toBe(expiredToken);
        });
        it('uses the cookies to get the token', function () {
            expect(authServer.getAccessToken({ cookies: cookies })).toBe('x' + expiredToken);
        });
        it('always prioritizes the headers', function () {
            expect(authServer.getAccessToken({ headers: headers, cookies: cookies, signedCookies: cookies })).toBe(expiredToken);
        });
        it('should be null with empty object', function () {
            expect(authServer.getAccessToken({})).toBe(null);
        });
        var _a;
    });
    describe('Gets a refreshToken from a request', function () {
        var signedCookies = (_a = {},
            _a[REFRESH_TOKEN_COOKIE] = 'xxx',
            _a);
        var cookies = (_b = {},
            _b[REFRESH_TOKEN_COOKIE] = 'yyy',
            _b);
        it('uses the signed cookies to get the token', function () {
            expect(authServer.getRefreshToken({ signedCookies: signedCookies })).toBe('xxx');
        });
        it('uses the cookies to get the token', function () {
            expect(authServer.getRefreshToken({ cookies: cookies })).toBe('yyy');
        });
        it('always prioritizes the signedCookies', function () {
            expect(authServer.getRefreshToken({ signedCookies: signedCookies, cookies: cookies })).toBe('xxx');
        });
        it('should be null with empty object', function () {
            expect(authServer.getRefreshToken({})).toBe(null);
        });
        var _a, _b;
    });
});
//# sourceMappingURL=auth.js.map