"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var scope_1 = require("./scope");
var AuthServer = /** @class */ (function () {
    function AuthServer(_a) {
        var AccessToken = _a.AccessToken, RefreshToken = _a.RefreshToken, payload = _a.payload, scope = _a.scope;
        this.accessToken = new AccessToken(this);
        this.refreshToken = new RefreshToken(this);
        this.payload = payload;
        this.scope = scope || new scope_1.default();
    }
    /**
     * Creates a new accessToken
     */
    AuthServer.prototype.createAccessToken = function (data) {
        var payload = this.accessToken.buildPayload(data);
        return {
            accessToken: this.accessToken.create(this.payload.create(payload)),
            payload: payload
        };
    };
    /**
     * Creates a new refreshToken
     */
    AuthServer.prototype.createRefreshToken = function (data) {
        return this.refreshToken.create(data);
    };
    /**
     * Creates both an accessToken and refreshToken
     */
    AuthServer.prototype.createTokens = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = {};
                        return [4 /*yield*/, this.createRefreshToken(data)];
                    case 1: return [2 /*return*/, __assign.apply(void 0, [(_a.refreshToken = _b.sent(), _a), this.createAccessToken(data)])];
                }
            });
        });
    };
    /**
     * Verifies an accessToken and returns its payload
     */
    AuthServer.prototype.verify = function (accessToken) {
        return this.payload.parse(this.accessToken.verify(accessToken));
    };
    /**
     * Decodes and returns the payload of an accessToken
     */
    AuthServer.prototype.decode = function (accessToken) {
        if (!accessToken)
            return null;
        try {
            return this.verify(accessToken);
        }
        catch (error) {
            return null;
        }
    };
    /**
     * Returns the payload in a refreshToken that can be used to create an
     * accessToken
     * @param reset Refresh the cookie of the refreshToken
     */
    AuthServer.prototype.getPayload = function (refreshToken, reset) {
        return this.refreshToken.getPayload(refreshToken, reset);
    };
    /**
     * Removes an active refreshToken
     */
    AuthServer.prototype.removeRefreshRoken = function (refreshToken) {
        if (refreshToken) {
            return this.refreshToken.remove(refreshToken);
        }
        return false;
    };
    /**
     * Returns a cookie, it searchs in this order:
     * signedCookies -> cookies -> null
     */
    AuthServer.prototype.getCookie = function (_a, cookie) {
        var cookies = _a.cookies, signedCookies = _a.signedCookies;
        return ((signedCookies && signedCookies[cookie]) ||
            (cookies && cookies[cookie]) ||
            null);
    };
    /**
     * Returns the accessToken from a JWT Token using the headers or cookies of a
     * request, it will always look inside headers first
     */
    AuthServer.prototype.getAccessToken = function (req) {
        var _a = (req.headers || {}).authorization, authorization = _a === void 0 ? '' : _a;
        var accessToken = (authorization &&
            typeof authorization === 'string' &&
            authorization.split(' ')[1]) ||
            this.getCookie(req, this.accessToken.cookie);
        return accessToken;
    };
    /**
     * Returns the refreshToken from the cookies
     */
    AuthServer.prototype.getRefreshToken = function (req) {
        return this.getCookie(req, this.refreshToken.cookie);
    };
    return AuthServer;
}());
exports.default = AuthServer;
//# sourceMappingURL=auth.js.map